;##########################################################################
;
;                          *** ct-BASIC ***
;
; EHBASIC extensions for T65 Core on FPGA by C. Meyer 3/2009
; Version 2.3 mit Hardware-Multiplier fr Textdarstellung
; 
; Im 6502 Simulator (Download unter http://home.pacbell.net/michal_k/) 
; als Code "ehbasic.dat" speichern mit:
; Dateityp "Binary image"
; Options: Begin $B000, End $FFFF
;
; C.Meyer 12/2009, cm@ctmagazin.de
;
;##########################################################################

	.START Reset
	.ROM_AREA $B200, $FFFF
	.OPT	Proc65c02, CaseInsensitive

; supported by t65 core: inc, dec, phx, plx, phy, ply
; missing: bra, ora, lda, cmp, sbc, tsb*2, trb*2, stz*2, bit*2, wai, stp, jmp, bbr*8, bbs*8

RealWorld	= 1	; luft auch im Simulator wenn = 0
GetRAMsize 	= 0	; RAM-Abfrage bei Start wenn = 1
LCD320		= 0	; Version fr LCD 320 x 240, 40 Zeichen / 20 zeilen

	.INCLUDE "defines_cm.asm"	; HW- und ZeroPage-Belegung
	
	; $B000 bis $B3FF frei fr Plot-Daten
	; $B000 bis $B0FF ScopeData
	*=$B000
	.INCLUDE "plotlib_cm.asm"
	; gleich anschlieend
	.INCLUDE "extensions_cm.asm" 	; darf $BFFF nicht berschreiten

	*=ROM_start	; immer bei $C000!

;Jump Table of important ROM routines	

	JMP 	Reset		; $C000, Cold Start
	JMP 	LAB_WARM	; $C003, do EhBASIC warm start
	JMP 	ClearVars	; $C006, Reset Vars, String Space etc.
	
	JMP 	AccToBank	; $C009, Memory Bank Management
	JMP 	BankToAcc  	; $C00C 

	JMP 	TickerDelay	; $C00F, VSync delay * Acc
	JMP 	Getkey		; der auf ZP $03 kann verbogen werden
	JMP 	Waitkey  		
	JMP 	WriteScreen		
	JMP 	WriteAcc		
	JMP 	ReceiveByte	; ber COMCH
	JMP 	SendByte	; ber COMCH
	JMP 	LAB_SCGB	; wichtig fr CALL Erweiterungen: ",<bytevar>"
; LAB_SCGI, wichtig fr CALL Erweiterungen: ",<intvar>", Ergebnis in Itempl/Itemph
	JSR	LAB_SCOMMA	; scan for "," , else do syntax error then warm start
; LAB_EVNM, wichtig fr CALL Erweiterungen, wie vor, jedoch ohne Komma
	JSR	LAB_EVNM	; evaluate expression and check is numeric,					; else do type mismatch
	JMP	LAB_F2FX	; save integer part of FAC1 in temporary integer

	.include "monitor_cm.asm"
	.INCLUDE "basic_cm.asm"

;#####################################################
;###             Memory Bank Access                ###
;#####################################################
	
; for Acc move from/to (BankPtrL),Y in Bank (AccessBank)
; BankPtrL = $52
; BankPtrH = $53
; AccessBank = $46, 70 dez.
; Benutzt X-Reg., Y als zus. Pointer (=0 wenn nicht gebraucht)

AccToBank:	
	LDX AccessBank
	STX Bank
	STA (BankPtr),Y
	JMP BankRestore
BankToAcc:
	LDX AccessBank
	STX Bank
	LDA (BankPtr),Y
BankRestore:
	LDX CurrBank
	STX Bank
	RTS


	.IF RealWorld

	

;############################################################
;#	 Zeichen-Ausgabe auf Bildschirm oder COM	    #
;############################################################

; Kommunikationsroutinen
	

omode_stx:		; sende STX (Start Text), warte auf ACK
	LDA #$80
	STA Omode
	LDA #2		; CTRL-B als Startzeichen senden
	JSR SendByte	; 
	LDA #4	
	JSR TickerDelay	; Verarbeitung abwarten
	RTS
	
omode_etx:
	LDA #$00
	STA Omode
	LDA #3		; CTRL-C/ETX als Ende-Zeichen-senden
	JMP SendByte	; nicht auf Screen, sondern Abschlusszeichen seriell

WriteAcc:		; Zeichen in Acc ausgeben
	CMP #$02	; STX? Omode auf AVR-COM/OptoBusCore, $02 senden
	BEQ omode_stx
	CMP #$03	; ETX? Omode auf Screen
	BEQ omode_etx	
	BIT Omode	; wenn $80, Ausgabe auf COM
	BPL WriteScreen
;	BVS WriteStatus	; Fehlermeldungen landen hier mit omode=$40

SendByte:		; Zeichen auf AVR-COM ausgeben
	STA asave
	STY ysave
	LDA COMCH
	BEQ SendByte0

SendByte1:		; SerialCore
	BIT UART1TxS
	BVS SendByte1	; warte bis Buffer nicht mehr voll
	LDA asave
	STA UART1D	; ausgeben
	LDY #0
Ydelay:		; 102 us Pause pro Byte
	DEY
	BNE Ydelay
	LDY ysave
	RTS
		
SendByte0:		; AVR COMport SPI
	BIT Tx0ACK
	BPL SendByte0	; noch kein ACK empfangen?
SendByte0weiter:
	LDA asave
	STA COMport
	LDY #$01	; Byte ausgeben, F_INT kurz triggern
	STY IRQena
	DEY
	STY IRQena	; ist 0
	JSR Ydelay
	CMP #$0D	; Acc=CR?
	BNE SendByte0exit
	LDA #0
	STA Tx0ACK	; ACK lschen nach CR
SendByte0exit:
	LDY ysave
	LDA asave		
	RTS

	

WriteStatus:		; Fehlermeldungen landen hier mit omode=$40

WriteScreen:		; Akku auf Screen, aktualisiert Cursor X/Y
	STA asave
	STY ysave
	STX xsave
; I/O-Steuerzeichen behandeln
	CMP #$1F	; writable?
	BMI WriteControls
writeChar:
	LDX InsFlg
	BEQ writeChar1
	JSR Insert
writeChar1:
	STA TextWR		; Zeichen in GTEXT-Schreibregister
	LDX CursX
	INC CursX
	CPX TextWinRight	; #colstotal	; Ende der Zeile?
	BEQ writeCharLF
	JMP restoreRegs
writeCharLF:	
	LDX TextWinLeft		; #0		; Zeilenanfang
	STX CursX
	JMP lf

WriteControls:
	JMP DispatchControls

restore:
	LDX CursX
	LDY CursY
	STX TextX
	STY TextY
restoreRegs:
	LDX xsave
	LDY ysave
	LDA asave
	RTS	

vt:	LDY CursY
	CPY TextWinTop	; #rowstart
	BEQ restore	; schon ganz oben?
	DEY
	STY CursY
	JMP restore

bs:	LDX CursX
	CPX TextWinLeft
	BEQ restore	; nicht unter 0
	DEX
	STX CursX	; beschreibt auch TextX
	JMP restore
	
ht:	LDX CursX
	INX
	CPX TextWinRight	; #colstotal-1
	BEQ restore	; nicht ber Ende der Zeile
	STX CursX	; beschreibt auch TextX
	JMP restore

lf:	LDY CursY
	INY
	STY CursY
	DEY
	CPY TextWinBtm	; #rowstotal
	BEQ scroll
	JMP restore

hc:	LDY TextWinTop	; #rowstart	; Home Cursor
	STY CursY
	LDX TextWinLeft	; #0
	STX CursX
	JMP restore

cr:	LDX #0
	STX InsFlg
	JSR InsertStatus
cr_0:
	LDX TextWinLeft	; #0
	STX CursX
cr_end:
	JMP restore

del:	LDX TextX
	BEQ del_end	; nicht unter 0
	DEX
	STX CursX
	LDA #$20	; mit Leerzeichen lschen
	STA TextWR
del_end:
	JMP restore
		
scroll:
	LDY TextWinTop	; #rowstart+1
scrollLoop:
	INY
	JSR LineCopyUp
	CPY TextWinBtm	; #rowstotal	; letzte Zeile?
	BNE scrollLoop
	JSR lineclrY
	STY CursY
	JMP restore
	
	
ClearFullScreen:
	STA asave
	STX xsave
	STY ysave
ClearFullScreen_0:
	JSR SetFullScreen
	JSR CursorOn
	LDY #rowstotal		; Mll in letzter Zeile+1
	JSR lineclrY
	
	LDA #$02		; CLS von Tastatur oder Reset immer auf grn
	STA Color
	LDA #$00
	STA GFrameC
	JSR FrameClear		; Grafik lschen
	
cls:	LDY #0		; oberste Zeile
	LDX #0
	.IF LCD320
	LDA #$01
	.ELSE
	LDA #7
	.ENDIF	
	STA ColorP
statusloop1:		; Statuszeile ausgeben
	LDA statusline1,X
	BEQ statusloop2	; String-Ende?
	JSR writecharxy
	INX 
	BNE statusloop1	; max. 255 Zeichen
statusloop2:		; Rest der Statuszeile lschen
	.IF LCD320
	.ELSE
statusbar:
	LDX #0
	INY
	LDA #202	; code page 437 "-"
statusbarloop:
	JSR writecharxy
	INX
	CPX #colstotal
	BNE statusbarloop
	.ENDIF
	
	JSR ClearWindow
	JSR CapsStatus
	JSR InsertStatus
	JSR COMstatus
	JSR BankStatus
	JSR ErrStatus
	JMP restore

ClearWindow:
	LDA Color	; letzte Grundfarbe
	STA ColorP
	LDA #$20
	LDY TextWinTop	; #rowstart 
clsloop:
	JSR lineclrY
	INY
	CPY TextWinBtm	; #rowstotal+1
	BNE clsloop
	JSR lineclrY
	LDX TextWinLeft
	STX CursX
	LDY TextWinTop	; #rowstart
	STY CursY
	RTS
			
Insert:		
; Bildschirmzeile ab Cursor eins nach rechts
	LDX TextWinRight
	CPX CursX	; schon am Ende der Zeile im Fenster?
	BEQ NoInsert
InsLoop:
	DEX
	STX TextX
	LDA TextWR
	INX
	STX TextX
	STA TextWR
	DEX
	CPX CursX		; An Cursorposition?
	BNE InsLoop
NoInsert:
	JMP restore


lineclrY:			
; Lscht Zeile in Y
	LDX TextWinLeft		; Ende der Zeile
lineclrFromX:
	STY TextY
	DEX
	LDA #$20		; Leerzeichen
lineclrloop:	
	INX
	STX TextX
	STA TextWR
	CPX TextWinRight
	BNE lineclrloop
	RTS

LineCopyUp:
; kopiert Zeile Y in Zeile Y-1
	STY TextY
	LDX TextWinRight
LineCopyUploop:
	STX TextX
	LDA TextWR
	DEY
	STY TextY
	STA TextWR
	INY
	STY TextY
	DEX
	CPX TextWinLeft	;#colstotal	; Ende der Zeile?
	BPL LineCopyUploop
	RTS

; Zeichen in A in Zeile X und Spalte Y schreiben, ohne Cursorposition zu verndern
WriteCharXY:
	STY TextY
	STX TextX
	STA TextWR		; Zeichen in GTEXT-Schreibregister
restoreTextXY:
	PHA
	LDA CursY
	STA TextY
	LDA CursX
	STA TextX
	PLA
	RTS
	
ReadCharXY:
	STY TextY
	STX TextX
	LDA TextWR		; Zeichen aus GTEXT-RAM
	JMP restoreTextXY
	
	
SetFullScreen:
;rowstart  = 2	; 
;rowstotal = 39	; 
;colstotal = 80	; 
	LDA #$00
	STA TextWinLeft
	STA CursX
	LDA #colstotal-1
	STA TextWinRight
	LDA #rowstart
	STA TextWinTop
	STA CursY
	LDA #rowstotal-1
	STA TextWinBtm
	RTS


 	.INCLUDE "basic_kbd.asm"	; Tastatur-Scancodes


; Byte in Acc vom FPGA-Modul SPI, von SerialCore oder OptoBusCore
; setzt Carry=1 wenn Byte empfangen, sonst Carry=0 und Acc=0

ReceiveByte:
	STY ytemp
; von AVRCOM- oder UART-IRQ im Buffer abgelegt
	LDY RxPtr	; prfen, ob etwas empfangen wurde
	CPY iRxPtr
	BEQ ReceiveByteExitNone	; nix da
	INY
	STY RxPtr
	LDA RxBuf,Y

ReceiveByteOK:
	LDY ytemp
	CMP #0
	SEC
	RTS

ReceiveByteExitNone:
	LDY ytemp
ReceiveStringError:
	LDA #0
	CLC
	RTS
	

; Universelle String-Input-Routine, behandelt CR/LF und LabReqIgnoreChr
; als Ende-Zeichen
; Fr LABINP-Antwort vom Modul setzt sie
; HashSignIdx	= $0716	; Position nach "#" in Ibuffs
; EqualSignIdx	= $0717 ; Position nach "=" in Ibuffs
; Setzt ein (von INPUT entferntes) <"> an den Ibuffs-Anfang $220

ReceiveString:
	LDA 	#15		; Timeout 0,25s (60Hz/6)
	STA 	TimeOut
	LDX	#0		; Pointer fr Ibuffs
	STX	HashSignIdx
	STX	EqualSignIdx
	STX	ColonSignIdx
	LDA	#'"'		; String-Input erwnscht, also mit "-Anfang simulieren
	STA	Ibuffs
	STA	Srchc
	INX
ReceiveString1:
	LDA	#0
	STA	Ibuffs,X	; Mit Ende-Null vorbelegen
	LDA	TimeOut
	BEQ	ReceiveStringError
	JSR	ReceiveByte
	BCC	ReceiveString1
	CMP	#$0A		; evt. brig von der letzten Antwort, ignorieren
	BEQ	ReceiveString1
	CMP 	#$0D		; CR?
	BEQ 	ReceiveStringEnded
	CMP 	LabReqIgnoreChr
	BEQ 	ReceiveStringRemaining
	CMP 	#'#'		; Startzeichen der Antwort?
	BNE 	ReceiveString2
	STX	HashSignIdx
	INC	HashSignIdx
ReceiveString2:
	CMP 	#':'		; Ergebnis der Antwort?
	BNE 	ReceiveString3
	STX	ColonSignIdx
	INC	ColonSignIdx
ReceiveString3:
	CMP 	#'='		; Ergebnis der Antwort?
	BNE 	ReceiveString4
	STX	EqualSignIdx
	INC	EqualSignIdx
ReceiveString4:
	CPX	#Ibuffe-Ibuffs	; compare character count with max
	BCS	ReceiveStringError	; exit if buffer full, Error!
	STA	Ibuffs,X	; else store in buffer
	INX
	BNE	ReceiveString1	; immer!
	BEQ	ReceiveStringError
	
ReceiveStringEnded:
	SEC
	RTS
	
ReceiveStringRemaining:
; Rest nach LabReqIgnoreChr abrufen (aber nicht speichern) bis CR kommt
	LDA	TimeOut
	BEQ	ReceiveStringError
	JSR	ReceiveByte
	BCC	ReceiveStringRemaining
	CMP 	#$0D		; CR?
	BNE	ReceiveStringRemaining
	LDX	#'X'-'0'		; "Extra Ignored (Rest of Line)"
	STX	LabErrFlags
	JSR 	ErrStatus
	JMP 	ReceiveStringEnded


TickerDelay:	; Timer-Wert in Acc, 60 => 1s
	STA	TimeOut
TickerDelayLoop:
	LDA	TimeOut	; NMI-Timer
	BNE	TickerDelayLoop
	RTS
	
	.ELSE

;############################################################################
; Testroutinen fr Simulator
; setup for the 6502 simulator environment

IO_AREA	= $8000		; set I/O area for this monitor
ACIAsimwr	= IO_AREA+$01	; simulated ACIA write port
ACIAsimrd	= IO_AREA+$04	; simulated ACIA read port

WriteAcc:		; Zeichen in Acc ausgeben
	BIT Omode	; wenn $80, Ausgabe auf COM
	BPL WriteScreen
SendByte:		; ersetzt PutByte
	PHA
	LDA #'^'
	STA ACIAsimwr	; save byte to simulated ACIA
	PLA
WriteScreen:
	STA ACIAsimwr	; save byte to simulated ACIA
	;JSR WriteScreen
	RTS

; byte in from simulated ACIA
ReceiveVal:
ReceiveByte:
GetKey:
	LDA ACIAsimrd	; get byte from simulated ACIA
	BEQ LAB_nobyw	; branch if no byte waiting
	SEC		; flag byte received
	RTS
LAB_nobyw
	CLC		; flag no byte received
	RTS
WaitKey:
	LDA ACIAsimrd	; get byte from simulated ACIA
	BEQ WaitKey	; branch if no byte waiting
	RTS
CapsToggle:
	RTS
Terminal:
	JSR Waitkey
	RTS
StartTTY:
BankStatus:
ClearFullScreen:
ClearWindow:
COMstatus:
writecharxy:
SetFullScreen:
TickerDelay:	; Timer-Wert in Acc, 60 => 1s
	RTS

	.ENDIF	



;############################################################################
;#                   OS-Routinen, Hilfsroutinen COM                         #
;############################################################################
	
VSwait:	; Delay 16ms
	BIT syncFlgs	; In flyback ?
	BPL VSwait	; ..yes, wait until finished
VSwait2:		; VSync Flyback
	BIT syncFlgs	; in flyback ?
	BMI VSwait2	; ..no, wait for flyback
	RTS
	
; does BASIC Timeout error
AVRCOM_ERR
	LDX	#$1E			; error code $1E ("Timeout" error)
	JMP	LAB_XERR		; do error #X, then warm start

	.IF RealWorld

	
;Warten bis Antwort vom AVR (intern) eintrifft	
WriteResponse:
	JSR	ReceiveString
	BCC	WriteResponseExit
	JSR	WriteIbuffS
	JMP	WriteResponse		; nochmal versuchen
WriteResponseExit:
	RTS
		
; prfen, ob es eine Antwort auf den Schirm zu schreiben gibt
GetResponse:
	JSR	ReceiveString 		;
	BCC	AVRCOM_ERR		; wenn timeout-Fehler
GetResponse1:
	CLC
WriteIbuffS:
	LDA	#<IBuffS+1		; Adresse des Strings +1 wg. "
	LDY	#>IBuffS
	JSR	LAB_18C3		; print null terminated string from memory
	JMP	LAB_CRLF		; print CR/LF and return
	
	
	.ELSE
WriteResponse:
GetResponse:
	RTS
	.ENDIF

; allg. Ausgabe an AVR

; Hilfsroutine zur Screen-Ausgabe eines Strings nach JSR WriteMsg
WriteMsg:		; Put Msg String Routine
	PLA		; Get the return address LSB from stack
	STA Cmdptr	; ..into w/s
	PLA		; Get the return address MSB from stack
	STA Cmdptr+1	; ..into w/s
WriteMsgChar:
	LDY #$0
	INC Cmdptr	; Point to the next byte of embedded text
	BNE WriteMsgNext
	INC Cmdptr+1	; Increment MSB text pointer if carried over
WriteMsgNext:
	LDA (Cmdptr),Y	; Get the next character of embedded text
	BMI WriteMsgExit; ..reached terminator - execute from here
	JSR WriteScreen	; Send character to Screen
	JMP WriteMsgChar; ..and go for the next character
WriteMsgExit:
	JMP (Cmdptr)	; Continue execution from the negative byte

; Hilfsroutine zur COM-Ausgabe eines Strings nach JSR PutMsg
SendMsg:		; Put Msg String Routine
	PLA		; Get the return address LSB from stack
	STA Cmdptr	; ..into w/s
	PLA		; Get the return address MSB from stack
	STA Cmdptr+1	; ..into w/s
SendMsgChar:
	LDY #$0
	INC Cmdptr	; Point to the next byte of embedded text
	BNE SendMsgNext
	INC Cmdptr+1	; Increment MSB text pointer if carried over
SendMsgNext:
	LDA (Cmdptr),Y	; Get the next character of embedded text
	BMI SendMsgExit	; ..reached terminator - execute from here
	JSR SendByte	; Send character to AVR
	JMP SendMsgChar	; ..and go for the next character
SendMsgExit:
	JMP (Cmdptr)	; Continue execution from the negative byte

; Integer-Ausgabe 16 Bit in A (MSB), X (LSB) auf Screen
WriteInteger =	LAB_295E	; print XA as unsigned integer

; Integer-Ausgabe 16 Bit in A (MSB), X (LSB) auf COM-Kanal
SendInteger:
	STA FAC1_1	; save high byte as FAC1 mantissa1
	STX FAC1_2	; save low byte as FAC1 mantissa2
	LDA #$00
	STA FAC1_3	; clear FAC1 mantissa3
	LDX #$90	; set exponent to 16d bits
	STX FAC1_e	; set FAC1 exponent
	BNE SendIntegerFAC	; immer
	
; Bank * $10000 plus Startadresse bermitteln
SendBankStart:
	LDA CurrBank	; Bank-Adresse (*$10000) addieren
	STA FAC2_1
	LDA #0
	STA FAC2_2
	STA FAC2_3
	STA FAC2_s
	STA FAC1_r
	LDA #$98
	STA FAC2_e
	JSR LAB_2474	; add FAC2 to FAC1
SendIntegerFAC:
	SEC		; set integer is +ve flag
	LDA #$00
	STA FAC1_r	; clear FAC1 rounding byte
	STA FAC1_s	; clear FAC1 sign (b7)
	JSR LAB_24D0	; do ABS and normalise FAC1
	LDY #$00	; clear index
	TYA		; clear A
	JSR LAB_297B	; convert FAC1 to string, skip sign character save
	JSR LAB_20AE	; print " terminated string to Sutill/Sutilh
	JSR LAB_22B6	; pop string off descriptor stack, or from top of string
	LDY #$00	; reset index
	TAX		; copy length to X
	BEQ SendIntegerExit	; exit (RTS) if null string
SendIntegerLoop:
	LDA (ut1_pl),Y	; get next byte
	JSR SendByte	; go output the character
	INY		; increment index
	DEX		; decrement count
	BNE SendIntegerLoop	; loop if not done
SendIntegerExit:
	RTS

	
SendInpStr:		; 0-Terminated-String im Input Buffer an COM bermitteln
	LDX #0
SendInpStrLoop:
	LDA Ibuffs,x
	BEQ SendInpStrExit
	JSR SendByte
	INX
	BNE SendInpStrLoop
SendInpStrExit:
	RTS

; String-Expression in BASIC auswerten und in Ibuffs kopieren	
EvaString:
	JSR	LAB_EVEX
	JSR	LAB_EVST	; evaluate string
	
; String in Ibuf kopieren, Lnge in A
	TAX
	LDY #0
EvaStringLoop:
	LDA (ut1_pl),Y		; get byte from string
	STA Ibuffs,Y
	INY
	DEX
	BNE EvaStringLoop
	LDA #0
	STA Ibuffs,Y		; 0-Terminieren
	RTS
	
COMswitch:
	LDA COMCH
	STA COMCHsave
	LDA #0
	STA COMCH
	JSR LAB_FlushRx		; COM Buffer leeren
	RTS

;#####################################################################
;#                 OS-Routinen, LOAD, DIR und SAVE                   #
;#            und weitere c't-LAB-BASIC-Erweiterungen                #
;#####################################################################

LAB_LABOUT:				; PRINT to COM, -cm
	PHA
;	JSR LAB_FlushRx
	LDA #$80
	STA Omode
	PLA
	JMP LAB_PRINT1			; geht ber WriteAcc

; neue LABREQ-Routine mit erweiterter Fehlerbehandlung

LAB_LABREQ: 				; perform LABREQ mainch, subch, result
	JSR 	LAB_FlushRx
	JSR 	LAB_LABCMDPREP
	LDA	#'['			; fhrt zu Abbruch/String-Ende
	STA	LabReqIgnoreChr
	LDA 	#'?'	
	JSR 	SendByte
	LDA 	#$0D	
	JSR 	SendByte
	JSR 	LAB_SCOMMA		; scan for ","
	JSR	ReceiveString
	BCC	LABINP_timeout		; zu lange gedauert
	LDX	HashSignIdx
	BEQ	LABINP_falseResponse	; kein "#" empfangen, also ganz daneben
	
; berprfung: Mit LabReqMainCh und LabReqSubCh bereinstimmend?
	LDA	LabReqMainCh
	CMP	#9			; Wildcard-Channel FPGA?
	BEQ	LAB_LABREQ0		; dann keine berprfung auf MainCh
	LDA	Ibuffs,X		; nur ein Zeichen fr MainCh, deshalb simpel:
	SEC
	SBC	#'0'
	CMP	LabReqMainCh
	BNE	LABINP_wrongMainCh
LAB_LABREQ0:
	LDX	EqualSignIdx
	BEQ	LABINP_falseResponse	; kein "=" empfangen, also ganz daneben
	LDX	ColonSignIdx
	BEQ	LABINP_falseResponse	; kein ":" empfangen, also ganz daneben
	JSR	LAB_IBufIdx2Itemp	; String in Integer
	BIT	ItempH
	BMI	LABINP_falseResponse	; wenn kein SubCh empfangen
	CMP	#255			; Acc: war es der Fehler-Kanal?
	BNE	LAB_LABREQ1
	LDX	EqualSignIdx		; ja: Fehlercode zurckgeben
	JSR	LAB_IBufIdx2Itemp	; String in Integer, LSB in Acc
	STA	LabErrFlags
	JSR 	ErrStatus
	JMP	LAB_LABREQ2
LAB_LABREQ1:
	CMP	LabReqSubCh
	BNE	LABINP_wrongSubCh	; sollte gleich dem angeforderten sein
; MainCh und SubCh stimmen jetzt mit den angeforderten berein
LAB_LABREQ2:
	LDA	EqualSignIdx		; hier beginnt der eigentliche Wert
		
LAB_LABINP2:
	CLC
	ADC	#<Ibuffs
	TAX
	LDY	#>Ibuffs		; set Y to buffer start-1 high byte	
	JMP	LAB_BUFINP		; weiter wie INPUT
	
	
; returns timeout error
LABINP_timeout
	LDX	#'T'-'0'		; "Timeout" error
	BNE	LABINP_StoreError	; do LAB error #X 

; unexpected answer received - Falscher MainCh als Antwort
LABINP_wrongMainCh:
	LDX	#'M'-'0'
	BNE	LABINP_StoreError	; do LAB error #X 

; unexpected answer received - Falscher MainCh als Antwort
LABINP_wrongSubCh:
	LDX	#'S'-'0'
	BNE	LABINP_StoreError	; do LAB error #X 
	
; unexpected answer received - String unvollstndig oder verstmmelt, kein "#"
LABINP_falseResponse:
	LDX	#'F'-'0'		; "False response"
LABINP_StoreError:
	STX	LabErrFlags
	JSR 	ErrStatus

;	JMP MonitorBoot
	JMP	LAB_DATA		; scan for next BASIC statement ([:] or [EOL])
	RTS				; and leave variable unchanged
	
	
; String im Ibuffs+X in Integer wandeln, LSB in Acc
LAB_IBufIdx2Itemp:
	LDA	#$80
	STA	ItempH
	LDA	Bpntrl		; BASIC-Zeiger retten
	PHA
	LDA	Bpntrh
	PHA
	LDA	#>Ibuffs	; Pointer auf Buffer-Zeile
	STA	Bpntrh
	TXA			; war Index
	CLC
	ADC	#<IBuffs	; Startadresse addieren
	STA	Bpntrl
	JSR	LAB_GBYT	; Byte holen und prfen: Ziffer?
	BCS	LAB_IBufIdx2Itemp0	
; Falls keine Ziffer, berspringen und Syntax Error vermeiden
	JSR	LAB_GFPN
LAB_IBufIdx2Itemp0:
	PLA
	STA	Bpntrh		; BASIC-Zeiger wiederherstellen
	PLA
	STA	Bpntrl
	LDA	ItempL
; Ergebnis jetzt in Acc und ItempL/ItempH mit ItempH=$80 wenn Fehler
	RTS
	
	
LAB_LABINP:	; LABINP Routine fr ct-LAB-Antworten, kompletter String
	LDA	#$00
	STA	LabReqIgnoreChr 
	STA	LabErrFlags		; Clear LABBUS Error
	SEC				; String 
	JSR	ReceiveString
	BCC	LABINP_timeout
	LDA	#0
	JMP	LAB_LABINP2		; weiter wie READ

; LABERR may be checked in BASIC to recognize false readings
LAB_LABERR:
	LDY	LabErrFlags	
	JMP	LAB_1FD0	; convert Y to byte in FAC1 and return

	
;############################################################################

; LabReqMainCh	= $0713 ; angeforderter MainCh
; LabReqSubCh	= $0714 ; angeforderter SubCh

LAB_LABCMDprep:
	JSR 	LAB_GTBY	; get byte parameter Main Channel in X
	STX	LabReqMainCh
	LDA 	#0
	JSR 	SendInteger
	LDA 	#':'	
	JSR 	SendByte
	JSR 	LAB_SCGB	; scan for "," and get SubChannel in X
	STX	LabReqSubCh
	LDA 	#0
	JMP 	SendInteger	; returns with RTS

LAB_LABCMD:
	JSR 	LAB_LABCMDprep
	LDA 	#'='	
	JSR 	SendByte
	JSR 	LAB_SCOMMA	; scan for ","
	JMP 	LAB_LABOUT
	

	
;############################################################################
;BorderLeft 	=  $07D8 ; Textfenster 0..79, 0..40 2004 dez.
;BorderTop 	=  $07D9 ; 2005 dez.
;BorderRight	=  $07DA ; 2006 dez.
;BorderBtm 	=  $07DB ; 2007 dez.

BorderLinksOben:
	.BYTE ' ',198,214
BorderRechtsOben:
	.BYTE ' ',204,220
BorderRechtsUnten:
	.BYTE ' ',201,217
BorderLinksUnten:
	.BYTE ' ',195,211
BorderHor:
	.byte ' ',202,218
BorderVert:
	.byte ' ',197,213
	
LAB_WINDOW_BORDER:
	JSR	LAB_SGBY	; get byte parameter
	STX	work
	LDA	BorderLinksOben,X
	LDY	BorderTop
	LDX	BorderLeft
	JSR	writecharxy
	LDX	work
	LDA	BorderRechtsOben,X
	LDX	BorderRight
	JSR	writecharxy
	LDX	work
	LDA	BorderRechtsUnten,X
	LDY	BorderBtm
	LDX	BorderRight
	JSR	writecharxy
	LDX	work
	LDA	BorderLinksUnten,X
	LDX	BorderLeft
	JSR	writecharxy
	JSR	LAB_WINDOW_BORDER_hor
		
LAB_WINDOW_BORDER_vert:	
	LDY	work
	LDA	BorderVert,Y	
	LDY	BorderTop
	INY
LAB_WINDOW_BORDER_vert_loop:
	LDX	BorderLeft
	JSR	writecharxy
	LDX	BorderRight
	JSR	writecharxy
	INY
	CPY	BorderBtm
	BNE	LAB_WINDOW_BORDER_vert_loop	
	RTS
	
LAB_WINDOW_BORDER_hor:
	LDX	work
	LDA	BorderHor,X	
	LDX	BorderLeft
	INX
LAB_WINDOW_BORDER_hor_loop:
	LDY	BorderBtm
	JSR	writecharxy
	LDY	BorderTop
	JSR	writecharxy
	INX
	CPX	BorderRight
	BNE	LAB_WINDOW_BORDER_hor_loop	
	RTS

LAB_WINDOW:
	CMP	#TK_CLEAR	; compare with token for CLEAR
	BEQ	LAB_WINDOW_CLEAR
	CMP	#TK_CURSXY	; compare with token for COLOR
	BEQ	LAB_WINDOW_CURSOR
	CMP	#TK_POSITION	; compare with token for POSition
	BEQ	LAB_WINDOW_POS
	CMP	#TK_BORDER	; compare with token for BORDER
	BEQ	LAB_WINDOW_BORDER
	CMP	#TK_OFF	; compare with token for OFF
	BEQ	LAB_WINDOW_OFF
	RTS	; Error
	
LAB_WINDOW_OFF
	JSR	LAB_IGBY	; just increment
	JMP 	SetFullScreen
	
LAB_WINDOW_POS:
	JSR	LAB_SGBY	; get byte parameter
	STX	TextWinLeft
	STX	CursX
	DEX
	STX	BorderLeft
	JSR	LAB_SCGB	; scan for "," and get byte parameter
	STX	TextWinTop
	STX	CursY
	DEX
	STX	BorderTop
	LDA	#TK_TO		; get "TO" token
	JSR	LAB_SCCA	; scan for CHR$(A) , else do syntax error then warm start
	JSR	LAB_GTBY	; get byte parameter
	STX	TextWinRight
	INX
	STX	BorderRight
	JSR	LAB_SCGB	; scan for "," and get byte parameter
	STX	TextWinBtm
	INX
	STX	BorderBtm
	RTS
	
LAB_WINDOW_CLEAR:
	JSR	LAB_IGBY	; just increment
	JMP	ClearWindow

	
LAB_WINDOW_CURSOR:		; Relative Cursor-Position innerhalb Fenster
	JSR	LAB_SGBY	; get byte parameter
	TXA
	CLC
	ADC	TextWinLeft
	STA	CursX
	JSR	LAB_SCGB	; scan for "," and get byte parameter
	TXA
	CLC
	ADC	TextWinTop
	STA	CursY
	RTS
	
	
LAB_COLOR:
	JSR	LAB_GTBY	; get byte parameter
	STX	Color
	STX	ColorP
	RTS

LAB_LABBUS:
	LDX	#0
	STX	UART_BAUD
	JSR	LAB_GTBY	; get byte parameter
	CPX	#$03		; compare with max
	BCS	LAB_CURSXY_FCER	; if greater do function call error
	STX	COMCH
	JSR	LAB_FlushRx
	STA 	UART1TxC	; auch UART-TxFIFO leeren
	DEX
	BMI	LAB_LABBUS0	; X war 0
	STX	UART_SEL
LAB_LABBUS0:
	JSR 	LAB_GBYT
	CMP 	#',' 	; Zweiter Parameter?
	BNE 	LAB_LABBUS_exit
	JSR	LAB_SCGB	; get "," and byte parameter
	CPX	#$08		; compare with max
	BCS	LAB_CURSXY_FCER	; if greater do function call error
	STX	UART_BAUD
LAB_LABBUS_exit:
	JSR 	ComStatus
	RTS			; return to BASIC

LAB_CURSXY:
	CMP	#TK_ON	; compare with token for ON
	BEQ	LAB_CURSOR_ON
	CMP	#TK_OFF	; compare with token for OFF
	BEQ	LAB_CURSOR_OFF
	JSR	LAB_GTBY	; get byte parameter
	STX	CursX
	JSR	LAB_SCGB	; scan for "," and get byte parameter
	STX	CursY
	RTS			; return to BASIC
	
LAB_CURSOR_ON:
	JSR	LAB_IGBY	; just increment
CursorOn:
	LDA #7
	STA CursorStyle
	RTS
	
LAB_CURSOR_OFF:
	JSR	LAB_IGBY	; just increment
	LDA #0
	STA CursorStyle
	RTS
	
; does BASIC function call error
LAB_CURSXY_FCER
	JMP	LAB_FCER	; do function call error, then warm start

SyntaxErr1:
	JMP	LAB_SNER	; do syntax error, then warm start

; Select Memory BANK command -cm
LAB_BANK
	JSR	LAB_GTBY	; get byte parameter
	CPX	#$08		; compare with max
	BCS	LAB_CURSXY_FCER	; if greater do function call error
LAB_BANK_0
	STX	Bank		; set new BANK
	STX	CurrBank
	STX	AccessBank
	JSR 	Bankstatus
	LDA	#$00
	STA	CapsFlg
	JSR	CapsToggle	; Grobuchstaben in BASIC
	JSR 	PScanSetBasParam	; BASIC-Parameter neu setzen, wie nach LOAD
	JMP 	0		; back to BASIC
	
LAB_FlushRx:			; Buffer leeren, benutzt nur Acc
	SEI
	STA 	UART1RxC	; UART-RxFIFO leeren
LAB_FlushRx0:
	LDA	UART1D
	BIT	UART1RxS
	BMI	LAB_FlushRx0
	LDA	#$00
	STA	LabReqIgnoreChr 
	STA 	iRxPtr		; Input Buffer leeren
	STA 	RxPtr
	LDA	#0
	STA	LabErrFlags	; Clear LABBUS Error
	JSR 	ErrStatus
	CLI
	RTS

LAB_SCANKEY:
	LDY	#0
	LDA	peekEvent	; neuer Event aufgetreten?
	BEQ	LAB_SCANKEY_exit
	LDY	peekScancode	; ja, Code zurckgeben
LAB_SCANKEY_exit	
	LDA	#0
	STA	peekEvent	
	JMP	LAB_1FD0	; convert Y to byte in FAC1 and return

SyntaxErr2:
	JMP	LAB_SNER	; do syntax error, then warm start

;############################################################################

LAB_CardDir:	; Directory on Screen
	JSR COMswitch
	JSR SendMsg
	.ASCII "9:DIR",$0D
	NOP
	JMP ResponseAndExit	; do EhBASIC warm start

LoadSavePrep:
; Default-AutoIncrement-Parameter setzen
	LDA iKbdPtr		; Kbd-buffer lschen
	STA KbdPtr

	JSR SendMsg
	.ASCII "9:AIS=0",$0D
	.ASCII "9:AIR=128",$0D	; Default-AutoIncRegister
	.ASCII "9:AIW=1",$0D
	.ASCII "9:AIM="	; Startadresse bermitteln
	NOP
	RTS
	
AccToFAC1:
	STA FAC1_1
	LDA #0
	STA FAC1_2
	STA FAC1_3
	LDA #$90	; Exponent fr 24 Bit
	STA FAC1_e
	RTS
	
LAB_SaveFile:
	JSR EvaString	; String in " " in IBuffs kopieren
	JSR COMswitch
	DEY
	BEQ SyntaxErr2
	JSR LoadSavePrep
	LDA smemh	; FAC1 mit BASIC-Startadresse $0800 vorbesetzen
	JSR AccToFAC1
	JSR SendBankStart
	JSR pscan	; Ende feststellen
	ldy #$00
	lda itempl
	sta (itempl),y
	iny
	lda itemph
	sta (itempl),y
	JSR SendMsg
	.ASCII $0D,"9:AIE="
	NOP
	INC itemph		; Page +1
	LDA itemph
	JSR AccToFAC1
	JSR SendBankStart
	JSR SendMsg	
	.ASCII $0D,"9:BSV=",34
	NOP
	JSR SendInpStr		; evaluated string
	JSR LAB_FlushRx
	JSR SendMsg	
	.ASCII 34,'!',$0D
	NOP
	LDA #$80	; ACK einschalten
	STA Tx0ACK
	
ResponseAndExit:
	JSR WriteResponse
NoResponseAndExit:
	JSR LAB_FlushRx
	LDA COMCHsave
	STA COMCH
	LDA CurrBank
	STA Bank
	JMP LAB_WARM	; do EhBASIC warm start
	
LAB_LoadFile		; load vector for EhBASIC
	JSR EvaString	; String in " " in IBuffs kopieren
	JSR LAB_GBYT
	CMP #',' 	; compare with comma
	BNE LoadnoParam
	JSR LAB_IGBY
	JSR LAB_EVNM	; evaluate expression and check is numeric, put into FAC1,
			; else do type mismatch
	JMP LoadFile	; nur laden
	
LoadnoParam:
	LDA smemh	; FAC1 mit BASIC-Startadresse $0800 vorbesetzen
	JSR AccToFAC1
	JSR LoadFile	
	JMP	Autostart
LoadFile:
	JSR COMswitch
	DEY
	BEQ SyntaxErr3
	JSR LoadSavePrep
; Integer-Ausgabe 24 Bit in CurrBank, Itempl/Itemph auf COM-Kanal
	JSR SendBankStart
	JSR SendMsg	
	.ASCII $0D,"9:BLD=",34 ; 
	NOP
	JSR SendInpStr		; evaluated string
	LDA	#5
	JSR 	TickerDelay
	JSR LAB_FlushRx
	JSR SendMsg	
	.ASCII 34,'!',$0D
	NOP
	LDA #$80	; FlowControl einschalten
	STA Tx0ACK
	JSR WriteResponse
	LDA CurrBank
	STA Bank
	LDA COMCHsave
	STA COMCH
	RTS
		
SyntaxErr3:
	JMP	LAB_SNER	; do syntax error, then warm start


LAB_MACRO:	; Type Textfile to SPI Kbd or execute .INI
	JSR EvaString	; String in " " in IBuffs kopieren
	JSR COMswitch	; String-Ende +1 jetzt in Y
	DEY
	BEQ SyntaxErr3
	TYA
	PHA 

	LDA iKbdPtr		; Kbd-buffer lschen
	STA KbdPtr
	JSR LAB_FlushRx

	JSR SendMsg	
	.ASCII "9:FQU=",34 	; File vorhanden?
	NOP
	JSR SendInpStr		; evaluated string
	JSR SendMsg	
	.ASCII 34,'!',$0D
	NOP
	JSR WriteResponse
	PLA
	TAY
	LDX #3			; Stringlnge-1	
LAB_MACRO_Compare:
	LDA	INIstring,Y
	CMP 	IbuffS,X 
	BNE 	TypeFile
	DEY
	DEX
	BPL	LAB_MACRO_Compare	
; muss jetzt Endung .INI haben
	JSR SendMsg	
	.ASCII "9:CFG=",34 ; 
	NOP
	JSR SendInpStr		; evaluated string
	JSR SendMsg	
	.ASCII 34,$0D
	NOP
	JMP NoResponseAndExit	; do EhBASIC warm start
	
TypeFile:
	JSR SendMsg
	.ASCII "9:TTF 67=",34 	; 
	NOP
	JSR SendInpStr		; evaluated string
	JSR SendMsg	
	.ASCII 34,$0D
	NOP
	JMP NoResponseAndExit	; do EhBASIC warm start

INIstring:
	.DB ".INI" 
	
;############################################################################
	
PScan	; Hilfsroutine: Suche nach Programm-Ende
	lda	smeml
      	sta	itempl
      	lda	smemh
      	sta	itemph
Pscan1	ldy	#$00
	lda	(itempl),y
	bne	pscan2
	iny   
	lda	(itempl),y
	bne	pscan2
	clc
	lda	#$02
	adc	itempl
	sta	itempl
	lda	#$00
	adc	itemph
	sta	itemph
	rts
Pscan2	ldy	#$00
	lda	(itempl),y
	tax
	iny
	lda	(itempl),y
	sta	itemph
	stx	itempl
	JMP	pscan1
	
PScanSetBasParam:
	JSR Pscan	; BASIC-Parameter neu setzen
	LDA itempl
	STA svarl
	STA sarryl
	STA earryl
	LDA itemph
	STA svarh
	STA sarryh
	STA earryh
	RTS
	
Autostart:
	JSR	PScanSetBasParam
	LDA RAM_base+5	; erster Zeile "REM AUTOSTART"?
	CMP #TK_REM
	BNE noAutoStart
	LDY #7		; Q&D: Absolute Adresse
	LDX #0
AutoSearchLoop:	
	LDA RAM_base,Y
	STA asave
	LDA AutoStartString,X
	CMP #0		; alles verglichen, Ende-Kennung
	BEQ AutoStartRun
	INY
	INX
	CMP asave
	BEQ AutoSearchLoop

	LDX #0
	JMP  LAB_warm			; 
	
AutoStartRun:
	LDA	#15	; 0,25s
	JSR	TickerDelay
	JMP  LAB_1477	; Autostart-RUN	
noAutoStart:
	JMP  LAB_warm			; Warmstart
;	JMP  LAB_1319			; Warmstart ohne "Ready"
AutoStartString:
	.DB "AUTOSTART",00  
	

;############################################################################
;############################################################################
;#        OS-Routinen, RST, IRQ-Aufbereitung und Initialisierung            #
;############################################################################
;############################################################################


Reset:
	SEI		; IRQ sperren bis IRQ-Routine ins RAM kopiert
	CLD		; clear decimal mode
	
	LDX #$FF	; empty stack
	TXS		; set the stack
	
	LDA	#<WriteAcc
	STA	WritejpL	; alten Write-Vektor wiederherstellen
	LDA	#>WriteAcc
	STA	WritejpH
	LDA	#<GetKey
	STA	GetkeyjpL	; alten GetKey-Vektor wiederherstellen
	LDA	#>GetKey
	STA	GetkeyjpH
	JSR	PlotInit1	; kehrt mit Acc=0 zurck
	STA COMCH
	STA IRQena
	STA iRxPtr
	STA RxPtr
	STA KbdPtr
	STA iKbdPtr
	STA CapsFlg	; UpCase-Flag, $20=Basic, $00=Terminal
	STA GetbFlg
	STA Omode	; 0 wenn PRINT, $80 wenn LABOUT auf COM
	STA Gsmall
	STA GFrameC	; Fenster-Hintergrund
	LDA IRQack
	LDA IRQclr
	LDA CurrBank
	STA Bank
	STA AccessBank	; fr PEEKs/POKEs
	STA	PS2sendStackPtr
	LDA #$80	; FlowControl einschalten
	STA Tx0ACK

	STA GsizeX
	LDA #50		; Grafik-Fenster
	STA GstartX
	LDA #36		; Grafik-Fenster
	STA GstartY
	LDA #$60	; Grafik-Fenster
	STA GsizeY
	
	JSR ClearFullScreen
	
	JSR CapsToggle
	JSR COMstatus
	STA Timer60
	CLI
	
; PS/2-Tastatur initialisieren	
	LDA	#$F6	
	STA	PS2sendStack+2
	LDA	#$ED	
	STA	PS2sendStack+1
	LDA	#$04		; CapsLock	
	STA	PS2sendStack
	LDA	#3
	STA 	PS2sendStackPtr	; immer zuletzt

; now do the signon message, Y = $00 here

LAB_signon
	.IF RealWorld
	LDA 	#10
	JSR	TickerDelay
	JSR 	COMswitch
	LDA 	#$0D
	JSR 	SendByte
	LDA 	#5
	JSR	TickerDelay
	JSR 	LAB_FlushRx
	LDA 	#$00
	STA 	PS2breakFlag
	STA 	PS2shiftFlag
	STA 	PS2ctrlFlag
	STA 	PS2altFlag
	STA 	PS2specialFlag
	STA	peekEvent
	LDA 	COMCHsave
	STA 	COMCH
	
	.endif
	JSR 	LAB_FlushRx
	JSR 	WriteMsg
	.BYTE $0D,$0D,$0A,"[C]old/[W]arm/[H]elp ?"	
	NOP
LAB_nokey
	JSR GetKey		; call scan input device
	BCC LAB_nokey		; loop if no key
	AND #$DF		; mask xx0x xxxx, ensure upper case
	CMP #'W'		; compare with [W]arm start
	BEQ LAB_dowarm		; branch if [W]arm start
	CMP #'H'		; compare with [W]arm start
	BEQ LAB_BASIChelp	; branch if [H]elp
 	CMP #'C'		; compare with [C]old start
	BNE LAB_nokey		; loop if not [C]old start
	LDA #0
	LDY #$07
ResetPageLoop:
	STY Bank
	STA RAM_base
	STA RAM_base+1
	STA RAM_base+2
	DEY
	BPL ResetPageLoop
	STA CurrBank
	STA AccessBank
	JSR Bankstatus
	JMP LAB_COLD		; do EhBASIC cold start
LAB_dowarm
	JSR Bankstatus
	JSR WriteScreen
LAB_release2
	JSR GetKey		; call scan input device
	BCS LAB_release2	; loop if key
	JMP LAB_WARM		; do EhBASIC warm start
TermJmp JSR Terminal
	JMP Reset	
MonJmp	JSR MonitorBoot
LAB_again:
	JMP Reset

LAB_BASIChelp:
	JSR 	ClearFullScreen
	JSR 	COMswitch
	JSR 	SendMsg
	.ASCII "9:TTY=BASHELP.TXT",$0D
	NOP
	JSR WriteResponse
	LDA COMCHsave
	STA COMCH
	JMP LAB_signon	; do EhBASIC warm start


;############################################################################
;#                            IRQ- und BRK-Handler                          #
;############################################################################

; BRK handler: Register retten und Monitor aufrufen

IRQentry:
	STA irqa	; hier mal nicht auf Stack
	LDA IRQsrc	; HW-Interrupt-Quelle vorrangig
	AND #$7F	; GDONE ausblenden
	STA irqs	; retten	
	BNE IRQnoBRK	; HW hat Vorrang
	PLA		; Get status
	PHA		; Re-save status
	AND #$10	; Is it IRQ or BRK ?
	BEQ IRQnoBRK
BRKentry:		; BRK Handler
	LDA irqa
	STA ACC		; fr Monitor
	STX Xreg	; save X for monitor
	STY Yreg	; save Y	
 	LDY IRQack
	LDY IRQclr	; IRQsrc gelesen, Latch lschen
	pla		; 
	STA Preg	; save P status
	PLA		; PCL
	STA PCL
	PLA		; PCH
	STA PCH
	TAX
	LDA PCL		; PCL
	SEC		; 2 subtrahieren fr echte BRK-Adresse
	SBC #$02	;
	STA PCL		; backup to BRK cmd
	BCS Brk2	;
	DEX		;
Brk2:	STX PCH		; save PC
	LDA irqa	; geretteter Acc
	TSX		; get stack pointer
	STX SPtr	; save stack pointer
	JSR PrintReg	; dump register contents
	LDX #$FF	; 
	TXS		; clear stack
	CLI		; enable interrupts again
	PLP
	PHP
	JMP Monitor	; start the monitor
	RTI
	
; CMs IRQ Handler: COM-, VS60Hz- und SPI-IRQ auswerten
IRQnoBRK:
	LDA irqa
	PHA
	TYA	; PHY fhrt hier zu sporadischen Fehlern!
	PHA
	LDY IRQack	; IRQ Reset
	LDY IRQclr	; IRQsrc gelesen, Latch lschen
; IRQs zuordnen: Bit 0 ist PS/2
IRQ_PS2:
	LSR 	irqs	; Saved IRQsrc
;	BEQ 	IRQexit	; sonst nix drin?	
	BCC 	IRQ_COM_JMP	; JMP auf IRQ_COM
; war Bit 0, also PS/2
	LDY	#$80		; fr zu setzende Flags
	LDA	PS2breakFlag
	BNE	IRQ_PS2_0	; es folgt nur der Loslass-Code
	
	LDA	PS2code		; Rohdaten vom Port
; Shift, Controls und Sondertasten umsetzen
	CMP	#$F0		; neuer Break-Code?
	BNE	IRQ_PS2_A
	STY	PS2breakFlag
	BEQ	IRQ_COM_JMP	; ignorieren
IRQ_PS2_A:
	CMP	#$FA		; Acknowledge?
	BEQ	IRQ_COM_JMP	; ignorieren
IRQ_PS2_B:
	CMP	#$E0		; Special Code?
	BNE	IRQ_PS2_C
	STY	PS2specialFlag
	BEQ	IRQ_COM
IRQ_PS2_C:
	BNE	IRQ_PS2_1	; immer

; Taste wurde losgelassen
IRQ_PS2_0:
	LDY	#0
	STY	PS2breakFlag	; lschen
	STY	PS2specialFlag	; lschen
	LDA	PS2code		; Rohdaten vom Port
	CMP	#$12
	BNE	IRQ_PS2_0A
	STY	PS2shiftFlag
	BEQ	IRQ_COM		; immer
IRQ_PS2_0A:
	CMP	#$59
	BNE	IRQ_PS2_0B
	STY	PS2shiftFlag
	BEQ	IRQ_COM		; immer
IRQ_PS2_0B:
	CMP	#$14
	BNE	IRQ_PS2_0C
	STY	PS2ctrlFlag
IRQ_PS2_0C:
; nichts weiter machen
IRQ_COM_JMP:
	JMP	IRQ_COM		; immer

; Taste wurde gedrckt
IRQ_PS2_1:
	LDA	PS2code		; Rohdaten vom Port
	CMP	#$12
	BNE	IRQ_PS2_1A
	STY	PS2shiftFlag
	BEQ	IRQ_COM		; immer
IRQ_PS2_1A:
	CMP	#$59
	BNE	IRQ_PS2_1B
	STY	PS2shiftFlag
	BEQ	IRQ_COM		; immer
IRQ_PS2_1B:
	CMP	#$14
	BNE	IRQ_PS2_1C
	STY	PS2ctrlFlag
	BEQ	IRQ_COM		; immer
IRQ_PS2_1C:
	
; neues Zeichen in Kbd-Buffer speichern
	STA	peekScancode
	TAY
	BIT 	PS2specialFlag		; E0-Codes?
	BMI 	IRQ_PS2_1D		; dann Controls nicht maskieren
	
	BIT 	PS2ctrlFlag		; Ctrl?
	BMI 	IRQ_PS2_maskControls
IRQ_PS2_1D:	
	ORA	PS2specialFlag
	STA	peekScancode
	BMI 	IRQ_PS2_storeBuf	; E0-Codes
	LDA 	PS2toASCII,Y		; Default: Unshifted
	BIT 	PS2shiftFlag
	BPL 	IRQ_PS2_storeBuf	; kein Shift?
	LDA 	PS2toASCIIshifted,Y	; Shifted Key
	JMP 	IRQ_PS2_storeBuf
IRQ_PS2_maskControls:
	LDA 	PS2toASCII,Y	; Default: Unshifted
	AND 	#$0F
IRQ_PS2_storeBuf:
	STY	peekEvent
	STA 	peekkey
;	STA 	$0600	;Test
	LDY 	iKbdPtr
	INY		; Zeiger erhhen, weil Byte eingetroffen
	STA 	KbdBuf,Y
	STY 	iKbdPtr	
		
; war Bit 1, also AVRCOM
IRQ_COM:
	LSR 	irqs	; Saved IRQsrc
	BCC	IRQ_UART
	LDA COMport	; COM-Byte von SPI eingetroffen
	BEQ IRQ_SPI	; Null-byte, nix machen
	CMP #$06
	BNE IRQ_COM_store
	LDA #$80	; ACK feststellen, aber nicht speichern
	STA Tx0ACK
	
; Bit 2, also UART?
IRQ_UART:
	LSR 	irqs	; Saved IRQsrc
	BCC	IRQ_SPI
	BIT UART1RxS	; Rx Status, Bit 7 = data present
	BPL IRQ_SPI	; Bit 7 = "0", nix mehr da
	LDA UART1D	; Byte abholen, setzt auch Rx_ack
	LDY iRxPtr
	INY		; Zeiger erhhen, weil Byte eingetroffen
	STA RxBuf,Y	; in Eingangspuffer
	STY iRxPtr
	JMP IRQ_UART	; noch was da?
;	JMP IRQ_KBD
	
IRQ_COM_store:
	LDY iRxPtr	; war AVRCOM oder UART
	INY		; Zeiger erhhen, weil Byte eingetroffen
	STA RxBuf,Y	; in Eingangspuffer
	STY iRxPtr

; Bit 3, also UART?
IRQ_SPI:
	LSR 	irqs	; Saved IRQsrc
	BCC	IRQexit		
	LDY iKbdPtr	; war SPI-Key
	LDA SPIkey	; Keyboard-Byte von SPI eingetroffen
	BEQ IRQexit	; Null-byte, nix machen
	INY		; Zeiger erhhen, weil Byte eingetroffen
	STA KbdBuf,Y
	STY iKbdPtr	
IRQexit:
	PLA
	TAY
	PLA
	RTI

	

; 60Hz Ticker NMI, reentrant zu IRQ
; PS2sendStackPtr	= $07E0	; Anzahl der PS/2-Bytes zum Gert (max. 5)
; PS2sendStack = $07E8	; 5 Bytes an PS/2, wird von oben abgearbeitet

NMIentry:
	PHA
	TYA
	PHA
;	LDA NMIack	; NMI angenommen
	LDY	PS2sendStackPtr
	BEQ	NMI_PS2sent	; alles gesendet?
	DEY
	LDA	PS2sendStack,Y
	STA	PS2code		; hat 1 Frame Zeit zum Senden
	STY	PS2sendStackPtr
NMI_PS2sent:	
	LDA 	TimeOut	
	BEQ 	NMInoDecTimer
	DEC 	TimeOut
NMInoDecTimer
	DEC 	Timer60	; war VSync
	PLA
	TAY
	PLA
	RTI

statusline1:
	.IF LCD320
	.db "EhBASIC #2.3g BANK   LABBUS            "
	.DB 0	; String-Ende, max. 39 Zeichen!
	.ELSE
	.db "c't-Lab EhBASIC Terminal #2.3g by cm        "  
	.db "BANK    LABERR    LABBUS            "
	.DB 0	; String-Ende, max. 79 Zeichen!
	.ENDIF

	
HelpTxt        .BYTE $0d, $0a
               .byte "x = HEX 4 char max~"
               .byte "x.x - Hexdump~"
               .byte "x:DD[ DD] - Change bytes~"
               .byte "xG - Go~"
               .byte "xL - List code~"
               .byte "x.x>xI - Move down~"
               .BYTE "x.x>xM - Move up~"
               .byte "x.xQ - Text dump~"
               .byte "R - Regs~"
               .byte "V - Version~"
               .byte "X - Exit~"
               .byte $00
Porttxt        .byte "6502 Mon v5.1"
               .byte $0d, $0a
               .byte "(? for help)"
               .byte $00

	*= $FFF4		; CALL -12 Terminal, -9 Monitor
	JMP Terminal
	JMP MonitorBoot
	
; system vectors
	*= $FFFA
SysVec:
	.word	NMIentry	; NMI vector
	.word	Reset		; RESET vector
	.word	IRQentry	; IRQ vector


	.END
